﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Net.Mail;
using System.Media;
using System.IO;
using System.Collections;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using nMail;

namespace AkaneMail
{
    public partial class Form1 : Form
    {
        // メールを格納する配列
        public ArrayList[] collectionMail = new ArrayList[3];

        // メールを識別する定数
        public const int RECEIVE = 0;   // 受信メール
        public const int SEND = 1;      // 送信メール
        public const int DELETE = 2;    // 削除メール

        // 選択された行を格納するフィールド
        private int currentRow;
        
        // 添付ファイルが存在するかのフラグ
        public bool attachMenuFlag = false;

        // 未読メールか既読メールかの確認フラグ
        public bool checkNotYetReadMail = false;

        // 環境保存用のクラスインスタンス
        private initClass initMail;

        // 環境設定保存のためのクラス
        public class initClass
        {
            public string m_fromName;       // ユーザ(差出人)の名前 
            public string m_mailAddress;    // ユーザのメールアドレス
            public string m_userName;       // ユーザ名
            public string m_passWord;       // POP3のパスワード
            public string m_popServer;      // POP3サーバ名
            public string m_smtpServer;     // SMTPサーバ名
            public int m_popPortNo;         // POP3のポート番号
            public int m_smtpPortNo;        // SMTPのポート番号
            public bool m_apopFlag;         // APOPのフラグ
            public bool m_deleteMail;       // POP受信時メール削除フラグ
            public bool m_popBeforeSMTP;    // POP before SMTPフラグ
            public bool m_autoMailFlag;     // メール自動受信フラグ
            public int m_getMailInterval;   // メール受信間隔
            public bool m_popSoundFlag;     // メール着信音フラグ
            public string m_popSoundName;   // メール着信音ファイル名
            public bool m_bodyIEShow;       // HTMLメール表示フラグ
        }

        // 点滅用 Win32API のインポート
        [DllImport("user32.dll")]
        private static extern bool FlashWindow(IntPtr hwnd, bool bInvert);
        public static void FlashWindow(System.Windows.Forms.Form window)
        {
            FlashWindow(window.Handle, false);
        }

        // ListViewの項目の並び替えに使用するクラス
        public class ListViewItemComparer : IComparer
        {
            private int _column;

            // ListViewItemComparerクラスのコンストラクタ
            // 並び替える列番号
            public ListViewItemComparer(int col)
            {
                _column = col;
            }

            // xがyより小さいときはマイナスの数、大きいときはプラスの数、
            // 同じときは0を返す
            public int Compare(object x, object y)
            {
                // ListViewItemの取得
                ListViewItem itemx = (ListViewItem)x;
                ListViewItem itemy = (ListViewItem)y;
                
                // xとyを文字列として比較する
                return string.Compare(itemx.SubItems[_column].Text, itemy.SubItems[_column].Text);
            }
        }

        public Form1()
        {
            InitializeComponent();

            // 配列を作成する
            collectionMail[RECEIVE] = new ArrayList();
            collectionMail[SEND] = new ArrayList();
            collectionMail[DELETE] = new ArrayList();
        }

        public void UpdateTreeView()
        {
            // メールの件数を設定する
            treeView1.Nodes[0].Nodes[0].Text = "受信メール (" + collectionMail[RECEIVE].Count + ")";
            treeView1.Nodes[0].Nodes[1].Text = "送信メール (" + collectionMail[SEND].Count + ")";
            treeView1.Nodes[0].Nodes[2].Text = "ごみ箱 (" + collectionMail[DELETE].Count + ")";
        }

        public void UpdateListView()
        {
            ArrayList list = null;

            // リストビューの内容をクリアする
            listView1.Items.Clear();

            if(listView1.Columns[0].Text == "差出人"){
                // 受信メールの場合
                list = collectionMail[RECEIVE];
            }
            else if(listView1.Columns[0].Text == "宛先"){
                // 送信メールの場合
                list = collectionMail[SEND];
            }
            else if(listView1.Columns[0].Text == "差出人または宛先"){
                // 削除メールの場合
                list = collectionMail[DELETE];
            }

            foreach (Mail mail in list){
                ListViewItem item = new ListViewItem(mail.address);
                if(mail.subject != ""){
                    item.SubItems.Add(mail.subject);
                }
                else{
                    item.SubItems.Add("(no subject)");
                }
                item.SubItems.Add(mail.date);
                item.SubItems.Add(mail.size);
                listView1.Items.Add(item);

                // 未読(未送信)の場合は、フォントを太字にする
                int index = listView1.Items.Count - 1;

                if (mail.notReadYet == true){
                    listView1.Items[index].Font = new Font(this.Font, FontStyle.Bold);
                }
            }
        }

        public void DirectSendMail(String address, String subject, String body, String attach)
        {
            try
            {
                // ステータスバーに状況表示する
                labelMessage.Text = "メール送信中";

                // POP before SMTPが有効の場合
                if(Mail.popBeforeSMTP == true){
                    // POP3のセッションを作成する
                    nMail.Pop3 pop = new nMail.Pop3();

                    // POP3への接続タイムアウト設定をする
                    nMail.Options.EnableConnectTimeout();

                    // APOPを使用するときに使うフラグ
                    pop.APop = Mail.apopFlag;

                    // POP3へ接続する
                    pop.Connect(Mail.popServer, Mail.popPortNumber);

                    // POP3への認証処理を行う
                    pop.Authenticate(Mail.userName, Mail.passWord);

                    // 何もせずに切断する
                    pop.Close();
                }

                // SMTPのセッションを作成する
                nMail.Smtp smtp = new nMail.Smtp(Mail.smtpServer);
                smtp.Port = Mail.smtpPortNumber;
                
                // 添付ファイルを指定している場合
                if(attach != ""){
                    smtp.FileName = attach;
                }
                
                // 追加ヘッダをつける
                smtp.Header = "\r\nX-Mailer:Ak@Ne 32bit Windows Mailler Version 1.01";

                // 差出人のアドレスを編集する
                string fromAddress = Mail.fromName + " <" + Mail.mailAddress + ">";

                // 送信する
                smtp.SendMail(address, fromAddress, subject, body);
                
                // SMTPから切断する
                smtp.Close();
                
                // ステータスバーに状況表示する
                labelMessage.Text = "メール送信完了";
            }
            catch (nMail.nMailException nex)
            {
                labelMessage.Text = "エラーNo:" + nex.ErrorCode + " エラーメッセージ:" + nex.Message; ;
            }
            catch (Exception exp)
            {
                labelMessage.Text = "エラーメッセージ:" + exp.Message;
            }
        }

        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (e.Node.Index == 0)
            {
                // 受信メールが選択された場合
                listView1.Columns[0].Text = "差出人";
                listView1.Columns[2].Text = "受信日時";
                if (e.Node.Text == "メールボックス")
                {
                    labelMessage.Text = "メールボックス";
                }
                else{
                    labelMessage.Text = "受信メール";
                }
            }
            else if (e.Node.Index == 1)
            {
                // 受信メールが選択された場合
                listView1.Columns[0].Text = "宛先";
                listView1.Columns[2].Text = "送信日時";
                labelMessage.Text = "送信メール";
            }
            else if (e.Node.Index == 2)
            {
                // 受信メールが選択された場合
                listView1.Columns[0].Text = "差出人または宛先";
                listView1.Columns[2].Text = "受信日時または送信日時";
                labelMessage.Text = "ごみ箱";
            }

            // リストビューを更新する
            UpdateListView();

        }

        private void menuFileExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void menuToolSetEnv_Click(object sender, EventArgs e)
        {
            Form2 Form2 = new Form2();

            if(timer2.Enabled == true){
                timer2.Enabled = false;
            }
            
            DialogResult ret = Form2.ShowDialog();

            if(ret == DialogResult.OK){
                if(Form2.checkAutGetMail.Checked == true){
                    timer2.Interval = Mail.getMailInterval * 60000;
                    timer2.Enabled = true;
                }
                else{
                    timer2.Enabled = false;
                }
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            labelDate.Text = DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToLongTimeString();

            // メールの選択数が0の場合
            if(listView1.SelectedItems.Count == 0){
                menuSaveMailFile.Enabled = false;
                menuMailDelete.Enabled = false;
                menuMailReturnMail.Enabled = false;
                menuDelete.Enabled = false;
                menuNotReadYet.Enabled = false;
                menuMailReurn.Enabled = false;
                buttonMailDelete.Enabled = false;
                buttonReturnMail.Enabled = false;
            }
            else{
                menuSaveMailFile.Enabled = true;
                menuMailDelete.Enabled = true;
                menuMailReturnMail.Enabled = true;
                menuDelete.Enabled = true;
                menuNotReadYet.Enabled = true;
                menuMailReurn.Enabled = true;
                buttonMailDelete.Enabled = true;
                buttonReturnMail.Enabled = true;
            }

            // 削除メールが0件の場合
            if(collectionMail[DELETE].Count == 0){
                menuClearTrush.Enabled = false;
                menuClearTrush_S.Enabled = false;
            }
            else{
                menuClearTrush.Enabled = true;
                menuClearTrush_S.Enabled = true;
            }

            // 添付ファイルが存在しない場合
            if(attachMenuFlag == false){
                menuFileGetAttatch.Enabled = false;
                menuGetAttach.Enabled = false;
            }
            else{
                if(listView1.SelectedItems.Count > 0){
                    menuFileGetAttatch.Enabled = true;
                    menuGetAttach.Enabled = true;
                }
                else{
                    menuFileGetAttatch.Enabled = false;
                    menuGetAttach.Enabled = false;
                }
            }

            // 未読メールの場合
            if(checkNotYetReadMail == true){
                menuNotReadYet.Enabled = false;
            }
            else{
                if(listView1.SelectedItems.Count > 0){
                    menuNotReadYet.Enabled = true;
                }
                else{
                    menuNotReadYet.Enabled =false;
                }
            }
        }

        private void menuMailNew_Click(object sender, EventArgs e)
        {
            String date = "未送信";
            bool notSendFlag = true;
            Form3 NewMailForm = new Form3();
            NewMailForm.ShowDialog();

            if(NewMailForm.resultSendMail == DialogResult.OK){
                // 直接送信を選択した場合
                if(NewMailForm.directSendFlag == true){
                    DirectSendMail(NewMailForm.textAddress.Text, NewMailForm.textSubject.Text, NewMailForm.textBody.Text, NewMailForm.attachName);
                    date = DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString();
                    notSendFlag = false;
                }

                // コレクションに追加する
                Mail newMail = new Mail(NewMailForm.textAddress.Text, "", NewMailForm.textSubject.Text, NewMailForm.textBody.Text, NewMailForm.attachName, date, "", "", notSendFlag);
                collectionMail[SEND].Add(newMail);

                // ツリービューとリストビューの表示を更新する
                UpdateTreeView();
                UpdateListView();
            }
        }

        private void menuMailSend_Click(object sender, EventArgs e)
        {
            try
            {
                // ステータスバーに状況表示する
                labelMessage.Text = "メール送信中";

                // POP before SMTPが有効の場合
                if(Mail.popBeforeSMTP == true){
                    // POP3のセッションを作成する
                    nMail.Pop3 pop = new nMail.Pop3();

                    // POP3への接続タイムアウト設定をする
                    nMail.Options.EnableConnectTimeout();

                    // APOPを使用するときに使うフラグ
                    pop.APop = Mail.apopFlag;

                    // POP3へ接続する
                    pop.Connect(Mail.popServer, Mail.popPortNumber);

                    // POP3への認証処理を行う
                    pop.Authenticate(Mail.userName, Mail.passWord);

                    // 何もせずに切断する
                    pop.Close();
                }

                // SMTPのセッションを作成する
                nMail.Smtp smtp = new nMail.Smtp(Mail.smtpServer);
                smtp.Port = Mail.smtpPortNumber;

                foreach (Mail mail in collectionMail[SEND]){
                    if(mail.notReadYet == true){
                        // 添付ファイルを指定している場合
                        if(mail.attach != ""){
                            smtp.FileName = mail.attach;
                        }

                        // 追加ヘッダをつける
                        smtp.Header = "\r\nX-Mailer:Ak@Ne 32bit Windows Mailler";

                        // 差出人のアドレスを編集する
                        string fromAddress = Mail.fromName + " <" + Mail.mailAddress + ">";

                        // 送信する
                        smtp.SendMail(mail.address, fromAddress, mail.subject, mail.body);

                        // 送信日時を設定する
                        mail.date = DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString();

                        // 送信済みに変更する
                        mail.notReadYet = false;
                    }
                }

                // SMTPから切断する
                smtp.Close();

                // ステータスバーに状況表示する
                labelMessage.Text = "メール送信完了";
            }
            catch (nMail.nMailException nex)
            {
                labelMessage.Text = "エラーNo:" + nex.ErrorCode + " エラーメッセージ:" + nex.Message; ;
            }
            catch (Exception exp)
            {
                labelMessage.Text = "エラーメッセージ:" + exp.Message;
            }

            // ツリービューとリストビューの表示を更新する
            UpdateTreeView();
            UpdateListView();
        }

        private void menuMailRecieve_Click(object sender, EventArgs e)
        {
            bool receivedFlag = false;      // 受信済みフラグ
            int mailCount = 0;              // 未受信メール件数

            try
            {
                // ステータスバーに状況表示する
                labelMessage.Text = "メール受信中";

                // POP3のセッションを作成する
                nMail.Pop3 pop = new nMail.Pop3();

                // POP3への接続タイムアウト設定をする
                nMail.Options.EnableConnectTimeout();

                // APOPを使用するときに使うフラグ
                pop.APop = Mail.apopFlag;

                // POP3へ接続する
                pop.Connect(Mail.popServer, Mail.popPortNumber);

                // POP3への認証処理を行う
                pop.Authenticate(Mail.userName, Mail.passWord);

                if(pop.Count >= 1){
                    // ステータスバーに状況表示する
                    labelMessage.Text = pop.Count + "件のメッセージがサーバ上にあります。";
                }
                else{
                    // ステータスバーに状況表示する
                    labelMessage.Text = "新着のメッセージはありませんでした。";
                }

                // 取得したメールをコレクションに追加する
                for(int no = 1; no <= pop.Count; no++){
                    // メールのUIDLを取得する
                    pop.GetUidl(no);

                    string uidl = pop.Uidl;

                    for(int i = 0; i < collectionMail[RECEIVE].Count; i++){
                        // uidlが受信済みメールの中に存在した場合
                        if(uidl == ((Mail)collectionMail[RECEIVE][i]).uidl){
                            receivedFlag = true;
                        }
                    }

                    // 未受信件数が1件以上の場合
                    if(pop.Count > 0){
                        // プログレスバーを表示して最大値を未受信メール件数に設定する
                        progressMail.Visible = true;
                        progressMail.Maximum = pop.Count;
                    }

                    // 受信済みでないメールの場合
                    if(receivedFlag == false){
                        // メールの情報を取得する
                        pop.GetMail(no);

                        // メールの情報を格納する
                        Mail mail = new Mail(pop.From, pop.Header, pop.Subject, pop.Body, pop.FileName, pop.DateString, pop.Size.ToString(), pop.Uidl, true);
                        collectionMail[RECEIVE].Add(mail);
                        mailCount++;
                        progressMail.Step = mailCount;
                        Application.DoEvents();
                    }

                    // メール削除フラグがtrueの場合メールを削除する
                    if(Mail.deleteMail == true){
                        pop.Delete(no);
                    }

                    // 受信済みフラグをfalseに戻す
                    receivedFlag = false;
                }

                // 未受信件数が1件以上の場合
                if (pop.Count > 0) {
                    // プログレスバーを非表示に戻す
                    progressMail.Visible = false;
                }

                // POP3から切断する
                pop.Close();

                // 未受信メールが1件以上の場合
                if(mailCount >= 1){
                    // メール着信音の設定をしている場合
                    if(Mail.popSoundFlag == true && Mail.popSoundName != ""){
                        SoundPlayer sndPlay = new SoundPlayer(Mail.popSoundName);
                        sndPlay.Play();
                    }

                    // 画面をフラッシュさせる
                    FlashWindow(this.Handle, false);

                    // ステータスバーに状況表示する
                    labelMessage.Text = mailCount + "件の新着メールを受信しました。";
                }
                else{
                    // ステータスバーに状況表示する
                    labelMessage.Text = "新着のメッセージはありませんでした。";
                }
            }
            catch (nMail.nMailException nex)
            {
                labelMessage.Text = "エラーNo:" + nex.ErrorCode + " エラーメッセージ:" + nex.Message;
            }
            catch (Exception exp)
            {
                // ステータスバーに状況表示する
                labelMessage.Text = "エラーメッセージ:" + exp.Message;
            }

            // ツリービューとリストビューの表示を更新する
            UpdateTreeView();
            UpdateListView();
        }

        private void menuDelete_Click(object sender, EventArgs e)
        {
            // 選択アイテムが0のときは反応にしない
            if(listView1.SelectedItems.Count == 0){
                return;
            }
            
            if(listView1.Columns[0].Text == "差出人"){
                // 受信メールの場合
                collectionMail[DELETE].Add(collectionMail[RECEIVE][currentRow]);
                collectionMail[RECEIVE].RemoveAt(currentRow);
                this.textBody.Text = "";
            }
            else if(listView1.Columns[0].Text == "宛先"){
                // 送信メールの場合
                collectionMail[DELETE].Add(collectionMail[SEND][currentRow]);
                collectionMail[SEND].RemoveAt(currentRow);
                this.textBody.Text = "";
            }
            else if(listView1.Columns[0].Text == "差出人または宛先"){
                // 削除メールの場合
                if(MessageBox.Show("選択されたメールは完全に削除されます。\nよろしいですか？", "確認", MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation) == DialogResult.OK){
                    collectionMail[DELETE].RemoveAt(currentRow);
                    this.textBody.Text = "";
                }
            }
            
            // ツリービューとリストビューの表示を更新する
            UpdateTreeView();
            UpdateListView();
        }

        private void menuNotReadYet_Click(object sender, EventArgs e)
        {
            // 選択アイテムが0のときは反応にしない
            if(listView1.SelectedItems.Count == 0){
                return;
            }
            
            if(listView1.Columns[0].Text == "差出人"){
                // 受信メールの場合
                ((Mail)collectionMail[RECEIVE][currentRow]).notReadYet = true;
            }
            else if(listView1.Columns[0].Text == "宛先"){
                // 送信メールの場合
                ((Mail)collectionMail[SEND][currentRow]).notReadYet = true;
            }
            else if(listView1.Columns[0].Text == "差出人または宛先"){
                // 削除メールの場合
                ((Mail)collectionMail[DELETE][currentRow]).notReadYet = true;
            }
            
            // ツリービューとリストビューの表示を更新する
            UpdateTreeView();
            UpdateListView();
        }

        private void listView1_ItemSelectionChanged(object sender, ListViewItemSelectionChangedEventArgs e)
        {
            // 選択された行を取得する
            currentRow = e.ItemIndex;
        }

        private void listView1_DoubleClick(object sender, EventArgs e)
        {
            Mail mail = null;

            if(listView1.Columns[0].Text == "差出人"){
                mail = (Mail)collectionMail[RECEIVE][currentRow];
            }
            else if (listView1.Columns[0].Text == "宛先"){
                mail = (Mail)collectionMail[SEND][currentRow];
            }
            else if (listView1.Columns[0].Text == "差出人または宛先"){
                mail = (Mail)collectionMail[DELETE][currentRow];
            }

            if(listView1.Columns[0].Text == "差出人" || listView1.Columns[0].Text == "差出人または宛先"){
                mail.notReadYet = false;
            }

            // ツリービューとリストビューの表示を更新する
            UpdateTreeView();
            UpdateListView();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            // ファイル展開用のテンポラリフォルダの削除
            if(Directory.Exists(Application.StartupPath + @"\tmp") == true){
                Directory.Delete(Application.StartupPath + @"\tmp", true);
            }

            // データファイルを削除する
            if(File.Exists(Application.StartupPath + @"Mail.dat") == true){
                File.Delete(Application.StartupPath + @"Mail.dat");
            }

            // ファイルストリームを作成する
            FileStream stream = new FileStream(Application.StartupPath + @"\Mail.dat", FileMode.Create);

            // ファイルストリームをストリームライタに関連付ける
            StreamWriter writer = new StreamWriter(stream, Encoding.Default);

            // メールの件数とデータを書き込む
            for(int i = 0; i < collectionMail.Length; i++){
                writer.WriteLine(collectionMail[i].Count.ToString());
                foreach(Mail mail in collectionMail[i]){
                    writer.WriteLine(mail.address);
                    writer.WriteLine(mail.subject);
                    writer.Write(mail.header);
                    writer.WriteLine("\x03");
                    writer.Write(mail.body);
                    writer.WriteLine("\x03");
                    writer.WriteLine(mail.date);
                    writer.WriteLine(mail.size);
                    writer.WriteLine(mail.uidl);
                    writer.WriteLine(mail.attach);
                    writer.WriteLine(mail.notReadYet.ToString());
                }
            }

            // ストリームライタとファイルストリームを閉じる
            writer.Close();
            stream.Close();

            // 環境設定保存クラスを作成する
            initMail = new initClass();

            // 環境設定ファイルへアカウント情報を保存する
            initMail.m_fromName = Mail.fromName;
            initMail.m_mailAddress = Mail.mailAddress;
            initMail.m_userName = Mail.userName;
            initMail.m_passWord = Mail.passWord;
            initMail.m_smtpServer = Mail.smtpServer;
            initMail.m_popServer = Mail.popServer;
            initMail.m_smtpPortNo = Mail.smtpPortNumber;
            initMail.m_popPortNo = Mail.popPortNumber;
            initMail.m_apopFlag = Mail.apopFlag;
            initMail.m_deleteMail = Mail.deleteMail;
            initMail.m_popBeforeSMTP = Mail.popBeforeSMTP;
            initMail.m_autoMailFlag = Mail.autoMailFlag;
            initMail.m_getMailInterval = Mail.getMailInterval;
            initMail.m_popSoundFlag = Mail.popSoundFlag;
            initMail.m_popSoundName = Mail.popSoundName;
            initMail.m_bodyIEShow = Mail.bodyIEShow;

            System.Xml.Serialization.XmlSerializer serializer = new System.Xml.Serialization.XmlSerializer(typeof(initClass));
            FileStream fs = new FileStream(Application.StartupPath + @"\AkaneMail.xml", FileMode.Create);
            serializer.Serialize(fs, initMail);
            fs.Close();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // 環境設定保存クラスを作成する
            initMail = new initClass();

            // アカウント情報(初期値)を設定する
            Mail.fromName = "";
            Mail.mailAddress = "";
            Mail.userName = "";
            Mail.passWord = "";
            Mail.smtpServer = "";
            Mail.popServer = "";
            Mail.popPortNumber = 110;
            Mail.smtpPortNumber = 25;
            Mail.apopFlag = false;
            Mail.deleteMail = false;
            Mail.popBeforeSMTP = false;
            Mail.autoMailFlag = false;
            Mail.getMailInterval = 10;
            Mail.popSoundName = "";
            Mail.bodyIEShow = false;

            // 環境設定ファイルが存在する場合は環境設定情報を読み込んでアカウント情報に設定する
            if(File.Exists(Application.StartupPath + @"\AkaneMail.xml")){
                System.Xml.Serialization.XmlSerializer serializer = new System.Xml.Serialization.XmlSerializer(typeof(initClass));
                FileStream fs = new FileStream(Application.StartupPath + @"\AkaneMail.xml", FileMode.Open);
                initMail = (initClass)serializer.Deserialize(fs);
                fs.Close();
                Mail.fromName = initMail.m_fromName;
                Mail.mailAddress = initMail.m_mailAddress;
                Mail.userName = initMail.m_userName;
                Mail.passWord = initMail.m_passWord;
                Mail.smtpServer = initMail.m_smtpServer;
                Mail.popServer = initMail.m_popServer;
                Mail.smtpPortNumber = initMail.m_smtpPortNo;
                Mail.popPortNumber = initMail.m_popPortNo;
                Mail.apopFlag = initMail.m_apopFlag;
                Mail.deleteMail = initMail.m_deleteMail;
                Mail.popBeforeSMTP = initMail.m_popBeforeSMTP;
                Mail.autoMailFlag = initMail.m_autoMailFlag;
                Mail.getMailInterval = initMail.m_getMailInterval;
                Mail.popSoundFlag = initMail.m_popSoundFlag;
                Mail.popSoundName = initMail.m_popSoundName;
                Mail.bodyIEShow = initMail.m_bodyIEShow;
            }

            // WinSockの初期化処理
            nMail.Winsock.Initialize();

            // ファイル展開用のテンポラリフォルダの作成
            if(Directory.Exists(Application.StartupPath + @"\tmp") == false){
                Directory.CreateDirectory(Application.StartupPath + @"\tmp");
            }

            if(File.Exists(Application.StartupPath + @"\Mail.dat") == true){
                // ファイルストリームを作成する
                FileStream stream = new FileStream(Application.StartupPath + @"\Mail.dat", FileMode.Open);

                // ファイルストリームをストリームリーダに関連付ける
                StreamReader reader = new StreamReader(stream, Encoding.Default);

                // GetHederFieldとHeaderプロパティを使うためPop3クラスを作成する
                nMail.Pop3 pop = new nMail.Pop3();

                // データを読み出す
                for (int i = 0; i < collectionMail.Length; i++){
                    // メールの件数を読み出す
                    int n = Int32.Parse(reader.ReadLine());

                    // メールを取得する
                    for(int j = 0; j < n; j++){
                        // 送信メールのみ必要な項目
                        string address = reader.ReadLine();
                        string subject = reader.ReadLine();
                        
                        // ヘッダを取得する
                        string header = "";
                        string hd = reader.ReadLine();

                        // 区切り文字が来るまで文字列を連結する
                        while(hd != "\x03"){
                            header = header + hd + "\r\n";
                            hd = reader.ReadLine();
                        }

                        // ヘッダのサイズが1バイト以上の場合
                        if(header.Length > 0){
                            // ヘッダープロパティにファイルから取得したヘッダを格納する
                            pop.Header = header;

                            // アドレスを取得する
                            pop.GetHeaderField("From:");
                            if(pop.Field != null){
                                address = pop.Field;
                            }

                            // 件名を取得する
                            pop.GetHeaderField("Subject:");
                            if(pop.Field != null){
                                subject = pop.Field;
                            }
                        }

                        // 本文を取得する
                        string body = "";
                        string b = reader.ReadLine();
                        
                        // 区切り文字が来るまで文字列を連結する
                        while (b != "\x03"){
                            body = body + b + "\r\n";
                            b = reader.ReadLine();
                        }

                        // 受信・送信日時を取得する
                        string date = reader.ReadLine();

                        // メールサイズを取得する(送信メールは無視)
                        string size = reader.ReadLine();

                        // UIDLを取得する(送信メールは無視)
                        string uidl = reader.ReadLine();

                        // 添付ファイル名を取得する(受信メールは無視)
                        string attach = reader.ReadLine();

                        // 既読・未読フラグを取得する
                        bool notReadYet = (reader.ReadLine() == "True");

                        // メール格納配列に格納する
                        Mail mail = new Mail(address, header, subject, body, attach, date, size, uidl, notReadYet);
                        collectionMail[i].Add(mail);
                    }
                }

                // ストリームリーダとファイルストリームを閉じる
                reader.Close();
                stream.Close();
            }

            // メール自動受信が設定されている場合はタイマーを起動する
            if(Mail.autoMailFlag == true){
                // 取得間隔*60000(60000秒=1分)をタイマー実行間隔に設定する
                timer2.Interval = Mail.getMailInterval * 60000;
                timer2.Enabled = true;
            }

            // ツリービューを展開する
            treeView1.ExpandAll();

            // ツリービューとリストビューの表示を更新する
            UpdateTreeView();
            UpdateListView();
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            // WinSockを開放する
            nMail.Winsock.Done();
        }

        private void menuMailReturnMail_Click(object sender, EventArgs e)
        {
            String date = "未送信";
            bool notSendFlag = true;
            Form3 NewMailForm = new Form3();

            // 選択アイテムが0のときは反応にしない
            if(listView1.SelectedItems.Count == 0){
                return;
            }

            NewMailForm.reMailFlag = true;
            NewMailForm.reAddress = ((Mail)collectionMail[RECEIVE][currentRow]).address;
            NewMailForm.reSubject = "Re:" + ((Mail)collectionMail[RECEIVE][currentRow]).subject;
            NewMailForm.reBody = "\r\n\r\n---" + ((Mail)collectionMail[RECEIVE][currentRow]).address + " was wrote ---\r\n" + ((Mail)collectionMail[RECEIVE][currentRow]).body;

            NewMailForm.ShowDialog();

            if(NewMailForm.resultSendMail == DialogResult.OK){
                // 直接送信を選択した場合
                if(NewMailForm.directSendFlag == true){
                    DirectSendMail(NewMailForm.textAddress.Text, NewMailForm.textSubject.Text, NewMailForm.textBody.Text, NewMailForm.attachName);
                    date = DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToShortTimeString();
                    notSendFlag = false;
                }

                // コレクションに追加する
                Mail newMail = new Mail(NewMailForm.textAddress.Text, "", NewMailForm.textSubject.Text, NewMailForm.textBody.Text, NewMailForm.attachName, date, "", "", notSendFlag);
                collectionMail[SEND].Add(newMail);

                // ツリービューとリストビューの表示を更新する
                UpdateTreeView();
                UpdateListView();
            }
        }

        private void listView1_Click(object sender, EventArgs e)
        {
            Mail mail = null;
            string bodyMail = "";
            Icon appIcon;
            bool htmlMail = false;

            // 表示機能はシンプルなものに変わる
            if(listView1.Columns[0].Text == "差出人"){
                mail = (Mail)collectionMail[RECEIVE][currentRow];
            }
            else if(listView1.Columns[0].Text == "宛先"){
                mail = (Mail)collectionMail[SEND][currentRow];
            }
            else if(listView1.Columns[0].Text == "差出人または宛先"){
                mail = (Mail)collectionMail[DELETE][currentRow];
            }

            // GetHederFieldとHeaderプロパティを使うためPop3クラスを作成する
            nMail.Pop3 pop = new nMail.Pop3();

            // ヘッダープロパティにファイルから取得したヘッダを格納する
            pop.Header = mail.header;

            // アドレスを取得する
            pop.GetHeaderField("Content-Type:");

            // Contents-Typeがtext/htmlのメールの場合
            if(pop.Field.Contains("text/html") == true){
                htmlMail = true;
            }
            else if(mail.body.Contains("<HTML") == true || mail.body.Contains("<html") == true){
                // multipart/alternativeで本文にHTMLタグがある場合
                htmlMail = true;
            }

            // 未読メールの場合
            if(mail.notReadYet == true) {
                checkNotYetReadMail = true;
            }
            else{
                checkNotYetReadMail = false;
            }

            // 添付ファイルクラスを作成する
            nMail.Attachment attach = new nMail.Attachment();

            // 今回は保存パスはプログラム直下に作成したtmpに設定する
            attach.Path = Application.StartupPath + @"\tmp";

            // ヘッダと本文付きの文字列を作成する
            bodyMail = mail.header + "\r\n" + mail.body;
            
            int id = attach.GetId(bodyMail);

            // 添付ファイルが存在する場合(存在しない場合は-1が返る)
            if(id != nMail.Attachment.NoAttachmentFile){
                try
                {
                    // ヘッダと本文付きの文字列を添付クラスに追加する
                    attach.Add(bodyMail);

                    // 添付ファイルを取り外す
                    attach.Save();
                }
                catch (Exception ex)
                {
                    // エラーメッセージを表示する
                    labelMessage.Text = String.Format("エラー メッセージ:{0:s}", ex.Message);
                }

                // 添付ファイルを外した本文が空値以外の場合
                if(attach.Body != ""){
                    if(attach.FileNameList != null){
                        // メニューに登録されている要素を破棄する
                        buttonAttachList.DropDownItems.Clear();
                        buttonAttachList.Visible = true;
                        attachMenuFlag = true;

                        // メニューに添付ファイルの名前を追加する
                        for(int no = 0; no < attach.FileNameList.Length; no++){
                            appIcon = System.Drawing.Icon.ExtractAssociatedIcon(Application.StartupPath + @"\tmp\" + attach.FileNameList[no]);
                            buttonAttachList.DropDownItems.Add(attach.FileNameList[no], appIcon.ToBitmap());
                        }
                    }
                    else{
                        // メニューに登録されている要素を破棄する
                        buttonAttachList.DropDownItems.Clear();
                        buttonAttachList.Visible = false;
                        attachMenuFlag = false;
                    }
                    if(htmlMail == false){
                        // 添付ファイルを外した本文をテキストボックスに表示する
                        this.browserBody.Visible = false;
                        this.textBody.Visible = true;
                        this.textBody.Text = attach.Body;
                    }
                    else{
                        // IEコンポーネントでメールを開く場合
                        if(Mail.bodyIEShow == true){
                            // テキストボックスを非表示にしてブラウザを表示に変更
                            this.textBody.Visible = false;
                            this.browserBody.Visible = true;

                            if(this.browserBody.Document == null){
                                this.browserBody.Navigate("about:blank");
                            }

                            // ブラウザにメールを表示する(添付側は簡易表示)
                            this.browserBody.Document.OpenNew(true);
                            this.browserBody.Document.Write(attach.Body);
                        }
                        else{
                            // 添付ファイルを外した本文をテキストボックスに表示する
                            this.browserBody.Visible = false;
                            this.textBody.Visible = true;
                            this.textBody.Text = attach.Body;
                        }
                    }
                }
                else{
                    // メニューに登録されている要素を破棄する
                    buttonAttachList.DropDownItems.Clear();
                    buttonAttachList.Visible = false;
                    attachMenuFlag = false;

                    // 本文をテキストボックスに表示する(過去に取得した中途半端なメールの救済措置)
                    this.browserBody.Visible = false;
                    this.textBody.Visible = true;
                    this.textBody.Text = mail.body;
                }
            }
            else{
                // 添付ファイルかないので添付ファイル一覧を削除する
                buttonAttachList.DropDownItems.Clear();
                buttonAttachList.Visible = false;
                attachMenuFlag = false;

                if(htmlMail == false){
                    // もともと添付ファイルが存在しない場合は本文をテキストボックスに表示する
                    this.browserBody.Visible = false;
                    this.textBody.Visible = true;
                    this.textBody.Text = mail.body;
                }
                else{
                    // IEコンポーネントでメールを開く場合
                    if (Mail.bodyIEShow == true) {
                        // テキストボックスを非表示にしてブラウザを表示に変更
                        this.textBody.Visible = false;
                        this.browserBody.Visible = true;

                        if(this.browserBody.Document == null){
                            this.browserBody.Navigate("about:blank");
                        }

                        // ブラウザにメールを表示する
                        this.browserBody.Document.OpenNew(true);
                        this.browserBody.Document.Write(mail.body);
                    }
                    else{
                        // 本文をテキストボックスに表示する
                        this.browserBody.Visible = false;
                        this.textBody.Visible = true;
                        this.textBody.Text = mail.body;
                    }
                }
            }
        }

        private void menuFileGetAttatch_Click(object sender, EventArgs e)
        {
            Mail mail = null;

            // 選択アイテムが0のときは反応にしない
            if(listView1.SelectedItems.Count == 0){
                return;
            }

            // 添付ファイル保存対象フォルダを選択する
            if(folderBrowserDialog1.ShowDialog() == DialogResult.OK){
                try
                {
                    // 添付ファイルクラスを作成する
                    nMail.Attachment attach = new nMail.Attachment();

                    // 保存パスを設定する
                    attach.Path = folderBrowserDialog1.SelectedPath;

                    // 対象は受信メールのみ
                    mail = (Mail)collectionMail[RECEIVE][currentRow];

                    // 添付ファイル展開用のテンポラリファイルを作成する
                    string tmpFileName = Path.GetTempFileName();
                    StreamWriter writer = new StreamWriter(tmpFileName);
                    writer.Write(mail.header);
                    writer.Write("\r\n");
                    writer.Write(mail.body);
                    writer.Close();

                    // テンポラリファイルを開いて添付ファイルを開く
                    StreamReader reader = new StreamReader(tmpFileName);
                    string header = reader.ReadToEnd();
                    reader.Close();

                    // ヘッダと本文付きの文字列を添付クラスに追加する
                    attach.Add(header);

                    // 添付ファイルを保存する
                    attach.Save();

                    MessageBox.Show(attach.Path + "に添付ファイル" + attach.FileName + "を保存しました。", "添付ファイルの取り出し", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                catch (nMailException nex)
                {
                    if (nex.ErrorCode == nMail.Attachment.ErrorFileOpen)
                    {
                        MessageBox.Show("添付ファイルがオープンできません。", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    }
                    else if (nex.ErrorCode == nMail.Attachment.ErrorInvalidNo)
                    {
                        MessageBox.Show("分割されたメールの順番が正しくないか、該当しないファイルが入っています。", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    }
                    else if (nex.ErrorCode == nMail.Attachment.ErrorPartial)
                    {
                        MessageBox.Show("分割されたメールが全て揃っていません", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show(String.Format("エラー メッセージ:{0:s}", ex.Message), "エラー", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                }
            }
        }

        private void menuSaveMailFile_Click(object sender, EventArgs e)
        {
            Mail mail = null;

            // 選択アイテムが0のときは反応にしない
            if(listView1.SelectedItems.Count == 0){
                return;
            }

            // 名前を付けて保存
            if(saveFileDialog1.ShowDialog() == DialogResult.OK){
                if(saveFileDialog1.FileName != ""){
                    try
                    {
                        mail = (Mail)collectionMail[RECEIVE][currentRow];
                        StreamWriter writer = new StreamWriter(saveFileDialog1.FileName);
                        writer.Write(mail.header);
                        writer.Write("\r\n");
                        writer.Write(mail.body);
                        writer.Close();
                    }
                    catch(Exception ex)
                    {
                        MessageBox.Show(String.Format("エラー メッセージ:{0:s}", ex.Message), "エラー", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    }
                }
            }
        }

        private void menuClearTrush_Click(object sender, EventArgs e)
        {
            int count = 0;

            count = collectionMail[DELETE].Count - 1;

            // ごみ箱の配列を空っぽにする
            if (MessageBox.Show("ごみ箱の中身をすべて削除します。\nよろしいですか？", "確認", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                for (int i = count; i >= 0; i--)
                {
                    collectionMail[DELETE].RemoveAt(i);
                }

                this.textBody.Text = "";

                // ツリービューとリストビューの表示を更新する
                UpdateTreeView();
                UpdateListView();
            }
        }

        private void menuHelpAbout_Click(object sender, EventArgs e)
        {
            // バージョン情報を表示する
            Form4 AboutForm = new Form4();
            AboutForm.ShowDialog();
        }

        private void buttonAttachList_DropDownItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            // ファイルを開くかの確認をする
            if(MessageBox.Show(e.ClickedItem.Text + "を開きますか？\nファイルによってはウイルスの可能性もあるため\n注意してファイルを開いてください。", "確認", MessageBoxButtons.YesNo, MessageBoxIcon.Information) == DialogResult.Yes){
                System.Diagnostics.Process.Start(Application.StartupPath + @"\tmp\" + e.ClickedItem.Text);
            }
        }

        private void listView1_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            //ListViewItemSorterを指定する
            listView1.ListViewItemSorter = new ListViewItemComparer(e.Column);
        }

        private void timer2_Tick(object sender, EventArgs e)
        {
            menuMailRecieve_Click(sender, e);
        }

    }
}